@@ -17,6 +17,7 @@ import sys
 import inspect
 import warnings
 import hashlib
+from http.client import HTTPMessage
 import logging
 import shlex
 import re
@@ -33,9 +34,7 @@ from urllib3 import exceptions
 logger = logging.getLogger(__name__)
 
 
-from botocore.vendored.six.moves import http_client
-
-class HTTPHeaders(http_client.HTTPMessage):
+class HTTPHeaders(HTTPMessage):
     pass
 
 from urllib.parse import (
